/******************************************************************************/
/*                                                                            */
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>

/********************************************************************* 
 * PEXlib Extended Pipeline Context Module:  Test Case 3  --  
 *                                           PEXExtChangePipelineContext
 *                                           TM Pipeline Context Attrs
 *                                           (using RGB colors)
 ********************************************************************/

test_image(Display         *dpy,
	   XID              resourceID,
           PEXOCRequestType req_type)
    {
    }

void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
    {
    strcpy(test_name, "pextmplc-1");
    *num_images      = 1;
    *supported_modes = NON_OC_MODE;

    strcpy(win_title, "pextmplc-1");
    }

void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
		int cur_image)
    {
    PEXRendererAttributes		rndAttrs, *rndAttrs2;
    unsigned long			item_mask = 0;
    unsigned long			valuemask[3];
    PEXPipelineContext                  plc;
    PEXExtPCAttributes			pc_values, *pc_values2;
    PEXTableIndex                       findex[4], bindex[5];

    _ibmpex_set_stderr("pextmplc-1.stderr", "pextmplc-1");

    valuemask[0] = 0;
    valuemask[1] = 0;
    valuemask[2] = 0;
    plc = PEXCreatePipelineContext(dpy, valuemask, &pc_values);

    PEXExtSetPCAttributeMask(valuemask, PEXExtPCTMPerspectiveCorrection);
    PEXExtSetPCAttributeMask(valuemask, PEXExtPCTMSampleFrequency);
    PEXExtSetPCAttributeMask(valuemask, PEXExtPCTMResourceHints);
    PEXExtSetPCAttributeMask(valuemask, PEXExtPCActiveTextures);
    PEXExtSetPCAttributeMask(valuemask, PEXExtPCBFActiveTextures);

    pc_values.tm_perspective_correction = PEXExtTMPerspCorrectVertex;
    pc_values.tm_sample_frequency = PEXExtTMSampleFrequencyInterpDep;

    pc_values.tm_resource_hints.optimization = PEXExtTMResourceHintSpace;
    pc_values.tm_resource_hints.prioritized_textures.count = 4;
    pc_values.tm_resource_hints.prioritized_textures.indices = findex;

    findex[0] = 1;
    findex[1] = 2;
    findex[2] = 3;
    findex[3] = 4;
    pc_values.active_textures.count = 4;
    pc_values.active_textures.indices = findex;

    bindex[0] = 1;
    bindex[1] = 2;
    bindex[2] = 3;
    bindex[3] = 4;
    bindex[4] = 5;
    pc_values.bf_active_textures.count = 5;
    pc_values.bf_active_textures.indices = bindex;
    
    PEXExtChangePipelineContext(dpy, plc, valuemask, &pc_values);

    valuemask[0] = 0;
    valuemask[1] = 0;
    valuemask[2] = 0;

    PEXExtSetPCAttributeMaskAll(valuemask);
    pc_values2 = PEXExtGetPipelineContext(dpy, plc, valuemask);
    if (pc_values2->tm_perspective_correction != 
                    PEXExtTMPerspCorrectVertex) {
      _ibmpex_stderr_print("ERROR 1 in PEXlib test pextmplc-1: PEXExtChangePipelineContext()\n");
       _ibmpex_stderr_print("tm_perspective_correction\n");
    }

    if (pc_values2->tm_sample_frequency != 
                    PEXExtTMSampleFrequencyInterpDep) {
      _ibmpex_stderr_print("ERROR 2 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("tm_sample_frequency\n");
    }

    if (pc_values2->tm_resource_hints.optimization != 
                    PEXExtTMResourceHintSpace) {
      _ibmpex_stderr_print("ERROR 3 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("tm_resource_hints.optimization\n");
    }

    if (pc_values2->tm_resource_hints.prioritized_textures.count !=4) {
      _ibmpex_stderr_print("ERROR 4 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("tm_resource_hints.prioritized_textures.count\n");
    }

    if (pc_values2->tm_resource_hints.prioritized_textures.indices[1] != 2) {
      _ibmpex_stderr_print("ERROR 4 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("tm_resource_hints.prioritized_textures.indices\n");
    }

    if (pc_values2->active_textures.count != 4 &&
        pc_values2->active_textures.indices[3] != findex[3]) {
      _ibmpex_stderr_print("ERROR 5 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("active_textures\n");
    }

    if (pc_values2->bf_active_textures.count != 5 &&
        pc_values2->bf_active_textures.indices[4] != bindex[4]) {
       _ibmpex_stderr_print("ERROR 6 in PEXlib test pextmplc-1:  PEXExtChangePipelineContext()\n");
      _ibmpex_stderr_print("bf_active_textures\n");
    }
         
    item_mask = (PEXRAPipelineContext | PEXRABackgroundColor | PEXRAClearImage);
    rndAttrs.pipeline_context = plc; 
    rndAttrs.background_color.type = PEXColorTypeRGB;
    rndAttrs.background_color.value.rgb.red = 0.0;
    rndAttrs.background_color.value.rgb.green = 0.0;
    rndAttrs.background_color.value.rgb.blue = 0.0;
    rndAttrs.clear_image = True;
    
    PEXChangeRenderer(dpy, renderer, item_mask, &rndAttrs);
    rndAttrs2 = PEXGetRendererAttributes(dpy, renderer, item_mask);
    if (plc != rndAttrs2->pipeline_context) {
      _ibmpex_stderr_print("ERROR 10 in PEXlib test pextmplc-1:  PEXEXTChangePipelineContext() or PEXGetRendererAttributes()\n");
    }

    PEXExtFreePCAttributes(pc_values2);
  }

void execute_test(Display *dpy, XID resourceID, 
	       PEXOCRequestType req_type, int cur_image)
    {
    describe_test("\nPipeline Context Test #3 (PEXExtChangePipelineContext)\n\n");
    describe_test(" This test, tests the above call with TextureMapping pipeline Context attributes\n");
    describe_test(" If correct, an empty stderr file is also produced.\n\n");

    test_image(dpy, resourceID, req_type);
    }

void testproc(Display *dpy, Window window, int cur_image)
    {
    file_testproc("pextmplc-1.stderr", "pextmplc-1");
     image_testproc("pextmplc-1", "pextmplc-1", img_desc,
		      dpy,window, 0, 0, 
		      glob_window_width, glob_window_height);
    }
